@@ -2,13 +2,14 @@
  * BRLTTY - A background process providing access to the console screen (when in
  *          text mode) for a blind person using a refreshable braille display.
  *
- * Copyright (C) 1995-2007 by The BRLTTY Developers.
+ * Copyright (C) 1995-2009 by The BRLTTY Developers.
  *
  * BRLTTY comes with ABSOLUTELY NO WARRANTY.
  *
  * This is free software, placed under the terms of the
  * GNU General Public License, as published by the Free Software
- * Foundation.  Please see the file COPYING for details.
+ * Foundation; either version 2 of the License, or (at your option) any
+ * later version. Please see the file LICENSE-GPL for details.
  *
  * Web Page: http://mielke.cc/brltty/
  *
@@ -29,10 +30,9 @@
 #include "program.h"
 #include "options.h"
 #include "brl.h"
-#include "tbl.h"
 #include "misc.h"
+#include "charset.h"
 #include "message.h"
-#include "scr.h"
 #include "defaults.h"
 
 int updateInterval = DEFAULT_UPDATE_INTERVAL;
@@ -80,33 +80,43 @@ BEGIN_OPTION_TABLE(programOptions)
   },
 END_OPTION_TABLE
 
-void
-message (const char *string, short flags) {
-  int length = strlen(string);
-  int limit = brl.x * brl.y;
+int
+message (const char *mode, const char *string, short flags) {
+  size_t length = strlen(string);
+  size_t size = brl.textColumns * brl.textRows;
+  char buffer[size];
 
   clearStatusCells(&brl);
 
-  memset(brl.buffer, ' ', brl.x*brl.y);
+  memset(buffer, ' ', size);
   while (length) {
-    int count = (length <= limit)? length: (limit - 1);
+    int count = (length <= size)? length: (size - 1);
     int index;
-    for (index=0; index<count; brl.buffer[index++]=*string++);
-    if (length -= count)
-      brl.buffer[(brl.x * brl.y) - 1] = '-';
-    else
-      while (index < limit) brl.buffer[index++] = ' ';
-    writeBrailleBuffer(&brl);
+
+    for (index=0; index<count; buffer[index++]=*string++);
+    if (length -= count) {
+      buffer[(count = size) - 1] = WC_C('-');
+    }
+
+    {
+      wchar_t characters[count];
+      wchar_t text[brl.textColumns * brl.textRows];
+      convertCharsToWchars(buffer, characters, count);
+      fillTextRegion(text, brl.buffer, 0, brl.textColumns, brl.textColumns, brl.textRows, characters, count);
+      if (!braille->writeWindow(&brl, text)) return 0;
+    }
 
     if (length) {
       int timer = 0;
-      while (braille->readCommand(&brl, BRL_CTX_MESSAGE) == EOF) {
+      while (braille->readCommand(&brl, BRL_CTX_WAITING) == EOF) {
         if (timer > 4000) break;
         approximateDelay(updateInterval);
         timer += updateInterval;
       }
     }
   }
+
+  return 1;
 }
 
 int
@@ -189,7 +199,6 @@ main (int argc, char *argv[]) {
     }
 
     if (chdir(opt_dataDirectory) != -1) {
-      makeUntextTable();
       initializeBrailleDisplay(&brl);
       brl.dataDirectory = opt_dataDirectory;
       identifyBrailleDriver(braille, 0);		/* start-up messages */
@@ -222,15 +231,17 @@ main (int argc, char *argv[]) {
   return status;
 }
 
-/* dummy function to allow brl.o to link... */
+/* dummy functions to allow drivers to link... */
+unsigned char
+cursorDots (void) {
+  return (BRL_DOT7 | BRL_DOT8);
+}
+
 void
 setHelpPageNumber (short page) {
 }
+
 int
 currentVirtualTerminal (void) {
   return 0;
 }
-int
-insertString (const char *string) {
-  return 0;
-}
