@@ -2,13 +2,14 @@
  * BRLTTY - A background process providing access to the console screen (when in
  *          text mode) for a blind person using a refreshable braille display.
  *
- * Copyright (C) 1995-2008 by The BRLTTY Developers.
+ * Copyright (C) 1995-2009 by The BRLTTY Developers.
  *
  * BRLTTY comes with ABSOLUTELY NO WARRANTY.
  *
  * This is free software, placed under the terms of the
  * GNU General Public License, as published by the Free Software
- * Foundation.  Please see the file COPYING for details.
+ * Foundation; either version 2 of the License, or (at your option) any
+ * later version. Please see the file LICENSE-GPL for details.
  *
  * Web Page: http://mielke.cc/brltty/
  *
@@ -29,42 +30,24 @@
 #include "options.h"
 #include "misc.h"
 #include "charset.h"
-#include "brl.h"
+#include "unicode.h"
+//#include "brl.h"
+#include "ttb.h"
 #include "ctb.h"
-#include "tbl.h"
 
-static char *opt_dataDirectory;
-static char *opt_contractionsDirectory;
 static char *opt_contractionTable;
 static char *opt_tablesDirectory;
 static char *opt_textTable;
 static char *opt_outputWidth;
+static int opt_forceOutput;
 
 BEGIN_OPTION_TABLE(programOptions)
-  { .letter = 'D',
-    .word = "data-directory",
-    .flags = OPT_Hidden,
-    .argument = "file",
-    .setting.string = &opt_dataDirectory,
-    .defaultSetting = DATA_DIRECTORY,
-    .description = "Path to directory for configuration files."
-  },
-
-  { .letter = 'C',
-    .word = "contractions-directory",
-    .flags = OPT_Hidden,
-    .argument = strtext("directory"),
-    .setting.string = &opt_contractionsDirectory,
-    .defaultSetting = DATA_DIRECTORY,
-    .description = strtext("Path to directory for contractions tables.")
-  },
-
   { .letter = 'c',
     .word = "contraction-table",
     .argument = "file",
     .setting.string = &opt_contractionTable,
     .defaultSetting = "en-us-g2",
-    .description = "Path to contraction table file."
+    .description = "Contraction table."
   },
 
   { .letter = 'T',
@@ -73,15 +56,14 @@ BEGIN_OPTION_TABLE(programOptions)
     .argument = strtext("directory"),
     .setting.string = &opt_tablesDirectory,
     .defaultSetting = DATA_DIRECTORY,
-    .description = strtext("Path to directory for text and attributes tables.")
+    .description = strtext("Path to directory containing tables.")
   },
 
   { .letter = 't',
     .word = "text-table",
     .argument = "file",
     .setting.string = &opt_textTable,
-    .defaultSetting = TEXT_TABLE,
-    .description = "Text translation table."
+    .description = "Text table."
   },
 
   { .letter = 'w',
@@ -91,16 +73,32 @@ BEGIN_OPTION_TABLE(programOptions)
     .defaultSetting = "",
     .description = "Maximum length of an output line."
   },
+
+  { .letter = 'f',
+    .word = "force-output",
+    .setting.flag = &opt_forceOutput,
+    .description = "Force immediate output."
+  },
 END_OPTION_TABLE
 
-TranslationTable textTable;
-TranslationTable untextTable;
 static ContractionTable *contractionTable;
 
 static int outputWidth;
 static int outputExtend;
 static unsigned char *outputBuffer;
 
+static void
+writeLocalCharacter (FILE *stream, unsigned char cell) {
+  fputc(convertDotsToCharacter(textTable, cell), stream);
+}
+
+static void
+writeUtf8Braille (FILE *stream, unsigned char cell) {
+  Utf8Buffer utf8;
+  size_t utfs = convertWcharToUtf8(cell|UNICODE_BRAILLE_ROW, utf8);
+  fprintf(stream, "%.*s", utfs, utf8);
+}
+
 typedef struct {
   unsigned char status;
 } LineProcessingData;
@@ -109,15 +107,15 @@ static int
 contractLine (char *line, void *data) {
   LineProcessingData *lpd = data;
   int lineLength = strlen(line);
-  wchar_t characters[lineLength];
-  const wchar_t *inputBuffer = characters;
+  wchar_t inputCharacters[lineLength];
+  const wchar_t *inputBuffer = inputCharacters;
 
   {
     int i;
-    for (i=0; i<lineLength; ++i) {
+    for (i=0; i<lineLength; i+=1) {
       wint_t character = convertCharToWchar(line[i]);
-      if (character == WEOF) character = 0XFFFD;
-      characters[i] = character;
+      if (character == WEOF) character = UNICODE_REPLACEMENT_CHARACTER;
+      inputCharacters[i] = character;
     }
   }
 
@@ -146,16 +144,19 @@ contractLine (char *line, void *data) {
       outputBuffer = NULL;
       outputWidth <<= 1;
     } else {
-      {
-        int index;
-        for (index=0; index<outputCount; ++index)
-          outputBuffer[index] = untextTable[outputBuffer[index]];
-      }
-
       {
         FILE *output = stdout;
-        fwrite(outputBuffer, 1, outputCount, output);
+        void (*writeCell) (FILE *stream, unsigned char cell) = textTable? writeLocalCharacter: writeUtf8Braille;
+        int index;
+
+        for (index=0; index<outputCount; index+=1) {
+          writeCell(output, outputBuffer[index]);
+          if (ferror(output)) break;
+        }
+
         if (!ferror(output)) fputc('\n', output);
+        if (opt_forceOutput && !ferror(output)) fflush(output);
+
         if (ferror(output)) {
           LogError("output");
           lpd->status = 12;
@@ -193,9 +194,7 @@ main (int argc, char *argv[]) {
 
   {
     char **const paths[] = {
-      &opt_dataDirectory,
       &opt_tablesDirectory,
-      &opt_contractionsDirectory,
       NULL
     };
     fixInstallPaths(paths);
@@ -213,18 +212,37 @@ main (int argc, char *argv[]) {
   outputBuffer = NULL;
 
   {
-    char *contractionTablePath;
+    char *contractionTableFile;
 
-    fixContractionTablePath(&opt_contractionTable);
-    if ((contractionTablePath = makePath(opt_contractionsDirectory, opt_contractionTable))) {
-      if ((contractionTable = compileContractionTable(contractionTablePath))) {
-        char *textTablePath;
+    if ((contractionTableFile = ensureContractionTableExtension(opt_contractionTable))) {
+      char *contractionTablePath;
 
-        fixTextTablePath(&opt_textTable);
-        if ((textTablePath = makePath(opt_tablesDirectory, opt_textTable))) {
-          if (loadTranslationTable(textTablePath, NULL, textTable, 0)) {
-            reverseTranslationTable(textTable, untextTable);
+      if ((contractionTablePath = makePath(opt_tablesDirectory, contractionTableFile))) {
+        if ((contractionTable = compileContractionTable(contractionTablePath))) {
+          if (*opt_textTable) {
+            char *textTableFile;
 
+            if ((textTableFile = ensureTextTableExtension(opt_textTable))) {
+              char *textTablePath;
+
+              if ((textTablePath = makePath(opt_tablesDirectory, textTableFile))) {
+                status = (textTable = compileTextTable(textTablePath))? 0: 4;
+
+                free(textTablePath);
+              } else {
+                status = 4;
+              }
+
+              free(textTableFile);
+            } else {
+              status = 4;
+            }
+          } else {
+            textTable = NULL;
+            status = 0;
+          }
+
+          if (!status) {
             if (argc) {
               do {
                 char *path = *argv;
@@ -245,20 +263,21 @@ main (int argc, char *argv[]) {
             } else {
               status = contractFile(stdin);
             }
-          } else {
-            status = 4;
+
+            if (textTable) destroyTextTable(textTable);
           }
-          free(textTablePath);
+
+          destroyContractionTable(contractionTable);
         } else {
-          status = 4;
+          status = 3;
         }
-        destroyContractionTable(contractionTable);
+
+        free(contractionTablePath);
       } else {
         status = 3;
       }
-      free(contractionTablePath);
-    } else {
-      status = 3;
+
+      free(contractionTableFile);
     }
   }
 
